
#include <winsock.h>
#include <stdio.h>
#include "include/mysql.h"

extern "C"
{
	// initializes MySQL
	MYSQL* bmx_mysql_init()
	{
		return mysql_init(NULL);
	}

	// closes the instance of MySQL
	void bmx_mysql_close(MYSQL* conn)
	{
		if (conn)
			mysql_close(conn);
	}

	// returns the number of the last error that occurred
	int bmx_mysql_errno(MYSQL* conn)
	{
		if (conn)
			return mysql_errno(conn);
		else return -1;
	}
	
	// returns the message of the last error that occurred
	const char * bmx_mysql_error(MYSQL* conn)
	{	
		if (conn)
			return mysql_error(conn);
		else return "";
	}

	// attempts to connect to a MySQL server
	bool bmx_mysql_connect(
		MYSQL* conn, 
		const char *host, 
		const char *user, 
		const char *passwd, 
		const char *db,
		unsigned int port, 
		const char *unix_socket=0,
		unsigned long clientflag=0)
	{
	
		bool retval=false;
		
		if (conn)
			retval=(mysql_real_connect(conn,host,user,passwd,db,port,unix_socket,clientflag)==conn); 
		
		return retval;
	}

	// selects the MySQL database on the server to use
	bool bmx_mysql_select_db(MYSQL* conn, const char *db)
	{
		if (conn)
			return (mysql_select_db(conn,db)==0);
		else return false;
	}

	// performs a query with binary support
	bool bmx_mysql_real_query(MYSQL* conn, const char *q, unsigned long length)
	{
		if (conn)
			return (mysql_real_query(conn,q,length)==0);
		else return false;
	}

	// returns the # of rows affected or returned.  use in place of mysql_num_rows().
	int bmx_mysql_affected_rows(MYSQL* conn) 
	{
		if (conn)
			return mysql_affected_rows(conn);
		else return 0;
	}

	int bmx_mysql_num_rows(MYSQL_RES* result)	
	{
		if (result)
			return mysql_num_rows(result);
		else return 0;
	}

	// returns the # of fields in the result
	int bmx_mysql_num_fields(MYSQL_RES* result)
	{
		if (result)
			return mysql_num_fields(result);
		else return -1;
	}
	
	// returns the length of the field
	int bmx_mysql_field_length(MYSQL_RES* result,unsigned int fieldnr)
	{
		if (result && fieldnr>=0)
			return mysql_fetch_lengths(result)[fieldnr];
		else return -1;

	}

	// can call for all queries.  returns 0 if no result set.  brings all rows down to client.
	MYSQL_RES* bmx_mysql_store_result(MYSQL* conn)
	{
		if (conn)
			return mysql_store_result(conn);
		else return NULL;
	}

	// same as mysql_store_result but keeps row data on server.
	MYSQL_RES* bmx_mysql_use_result(MYSQL* conn)
	{
		if (conn)
			return mysql_use_result(conn);
		else return NULL;
	}

	// returns the next row.  returns NULL when done or fails.
	MYSQL_ROW bmx_mysql_fetch_row(MYSQL_RES* result) // MYSQL_ROW is really a char** (array of strings)
	{
		if (result)			
			return mysql_fetch_row(result);
		return NULL;
	}

	bool bmx_mysql_eof(MYSQL_RES* result)
	{
		if (result)
			return mysql_eof(result);
		else return true;
	}

	int bmx_mysql_insert_id(MYSQL* conn)
	{
		if (conn)
			return mysql_insert_id(conn);
		else return -1;
	}
	
	void bmx_mysql_free_result(MYSQL_RES* result)
	{
		if (result)
			mysql_free_result(result);
	}

	void bmx_mysql_data_seek(MYSQL_RES* result, my_ulonglong offset)
	{
		if (result)
			mysql_data_seek(result,offset);
	}

	const char* bmx_mysql_field_name(MYSQL_RES* result, unsigned int fieldnr)
	{
		if (result && fieldnr>=0)
			return mysql_fetch_field_direct(result,fieldnr)->name;
		else return NULL;
	}

	// returns the value of a field
	const char* bmx_mysql_field_value(MYSQL_ROW row,unsigned int fieldnr)
	{
		if (row && fieldnr>=0)
			return row[fieldnr];
		else return NULL;
	}

	bool bmx_mysql_isfieldnull(MYSQL_ROW row,unsigned int fieldnr)
	{
		return (bmx_mysql_field_value(row,fieldnr)==NULL);
	}

	int bmx_mysql_real_escape_string(MYSQL* conn, char *to, const char *from, unsigned long length)
	{
		if (conn)
			return mysql_real_escape_string(conn, to, from, length);
		else return -1;
	}
/*




// NOTE: fetching interface (or any looping) needs to have a hook for status (ie notification interface type that can be hooked)

MYSQL_FIELD *mysql_fetch_field_direct(MYSQL_RES *result, unsigned int fieldnr)


typedef struct st_mysql_field {
  char *name;                 // Name of column 
  char *org_name;             // Original column name, if an alias  
  char *table;                // Table of column if column was a field 
  char *org_table;            // Org table name, if table was an alias 
  char *db;                   // Database for table 
  char *catalog;	      // Catalog for table 
  char *def;                  // Default value (set by mysql_list_fields) 
  unsigned long length;       // Width of column (create length) 
  unsigned long max_length;   // Max width for selected set 
  unsigned int name_length;
  unsigned int org_name_length;
  unsigned int table_length;
  unsigned int org_table_length;
  unsigned int db_length;
  unsigned int catalog_length;
  unsigned int def_length;
  unsigned int flags;         // Div flags 
  unsigned int decimals;      // Number of decimals in field 
  unsigned int charsetnr;     // Character set 
  enum enum_field_types type; // Type of field. See mysql_com.h for types 
} MYSQL_FIELD;

typedef struct st_mysql
{
  NET		net;			// Communication parameters 
  gptr		connector_fd;		// ConnectorFd for SSL 
  char		*host,*user,*passwd,*unix_socket,*server_version,*host_info,*info;
  char          *db;
  struct charset_info_st *charset;
  MYSQL_FIELD	*fields;
  MEM_ROOT	field_alloc;
  my_ulonglong affected_rows;
  my_ulonglong insert_id;		// id if insert on table with NEXTNR 
  my_ulonglong extra_info;		// Used by mysqlshow 
  unsigned long thread_id;		// Id for connection in server 
  unsigned long packet_length;
  unsigned int	port;
  unsigned long client_flag,server_capabilities;
  unsigned int	protocol_version;
  unsigned int	field_count;
  unsigned int 	server_status;
  unsigned int  server_language;
  unsigned int	warning_count;
  struct st_mysql_options options;
  enum mysql_status status;
  my_bool	free_me;		// If free in mysql_close 
  my_bool	reconnect;		// set to 1 if automatic reconnect 

  // session-wide random string 
  char	        scramble[SCRAMBLE_LENGTH+1];

 //
   Set if this is the original connection, not a master or a slave we have
   added though mysql_rpl_probe() or mysql_set_master()/ mysql_add_slave()
 
  my_bool rpl_pivot;
  //
    Pointers to the master, and the next slave connections, points to
    itself if lone connection.
  
  struct st_mysql* master, *next_slave;

  struct st_mysql* last_used_slave; // needed for round-robin slave pick 
 // needed for send/read/store/use result to work correctly with replication 
  struct st_mysql* last_used_con;

  LIST  *stmts;                     // list of all statements 
  const struct st_mysql_methods *methods;
  void *thd;
  //
    Points to boolean flag in MYSQL_RES  or MYSQL_STMT. We set this flag 
    from mysql_stmt_close if close had to cancel result set of this object.
  
  my_bool *unbuffered_fetch_owner;
} MYSQL;

typedef struct st_mysql_res {
  my_ulonglong row_count;
  MYSQL_FIELD	*fields;
  MYSQL_DATA	*data;
  MYSQL_ROWS	*data_cursor;
  unsigned long *lengths;		// column lengths of current row 
  MYSQL		*handle;		// for unbuffered reads 
  MEM_ROOT	field_alloc;
  unsigned int	field_count, current_field;
  MYSQL_ROW	row;			// If unbuffered read 
  MYSQL_ROW	current_row;		// buffer to current row 
  my_bool	eof;			// Used by mysql_fetch_row 
  // mysql_stmt_close() had to cancel this result 
  my_bool       unbuffered_fetch_cancelled;  
  const struct st_mysql_methods *methods;
} MYSQL_RES;

typedef struct st_mysql_data {
  my_ulonglong rows;
  unsigned int fields;
  MYSQL_ROWS *data;
  MEM_ROOT alloc;
#if !defined(CHECK_EMBEDDED_DIFFERENCES) || defined(EMBEDDED_LIBRARY)
  MYSQL_ROWS **prev_ptr;
#endif
} MYSQL_DATA;
*/

}

